/*------------------------------------------------------------------------------
  ().........:   igor727@mail.ru
  ..: 
  .......:
  .........:        QN8027  RDS    vrtp.ru
  .............:
------------------------------------------------------------------------------*/
#include <ioavr.h>
#include <intrinsics.h>
#include "i2c.h"
#include "QN8027.h"
#include "Settings.h"
//==============================================================================
unsigned char         QN8027_System;
unsigned int          FreqCh;
unsigned char         FDev; 
extern unsigned char  Mute;
extern unsigned char  MicGain;
unsigned char         RDS_Counter; 
unsigned char         RDS_Enable;     // 1- RDS , 0 - 
unsigned char         TxPdClr;        //     TXPDCLR
unsigned char         AGCReturnTimeCounter; //     
unsigned char         AGCMicGain;     //    
unsigned char         AGCEnable;

//==============================================================================
//============================== QN8027_SendData ===============================
//==============================================================================
void QN8027_SendData (unsigned char adr_register, unsigned char data)
{ //    "data"  QN8027     "adr_register"
  i2c_start ();
  i2c_send_byte (QN8027W_ADR);
  i2c_send_byte (adr_register);
  i2c_send_byte (data);
  i2c_stop ();
}

//==============================================================================
//============================== QN8027_ReadData ===============================
//==============================================================================
unsigned char QN8027_ReadData (unsigned char adr_register)
{ //    QN8027   "adr_register"
  unsigned char ret;
  i2c_start ();
  i2c_send_byte (QN8027W_ADR);
  i2c_send_byte (adr_register); 
  i2c_stop ();
  //
  i2c_start ();
  i2c_send_byte (QN8027R_ADR);
  ret = i2c_read_byte(NACK);
  i2c_stop (); 
  return ret;
}




//==============================================================================
//================================= QN8027_Init ================================
//==============================================================================
void QN8027_Init (void)
{ //  QN8027
  _delay_ms (QNDELAY);
  QN8027_SendData (0x00, 0x80);           //   
  _delay_ms (QNDELAY);
  QN8027_SendData (0x00, 0x40);
  _delay_ms (QNDELAY);
  // 
  QN8027_System = ((1<<5) | (MONO<<4)) | ((FreqCh >> 8) & 0x03);  
  QN8027_SendData (0x00, QN8027_System);
  QN8027_SendData (0x01, FreqCh); 
  QN8027_SendData (0x02, (TC<<7) | GAIN_TXPLT); 
  QN8027_SendData (0x03, XISEL); 
  QN8027_SendData (0x04, XSEL<<7);                    // 0x04
  QN8027_SendData (0x10, POWER);                      // 0x10
  QN8027_SendData (0x12, (RDS_Enable <<7)|(RDS_DEV)); // 0x12    
  QN8027_SetDeviation ();
  QN8027_SetMicGain (MicGain);
}

//==============================================================================
//============================== QN8027_MuteOn =================================
//==============================================================================
void QN8027_MuteOn (void)
{ //  
  if (Mute==0)
  { QN8027_System |= (1<<3);            // .  Mute
    Mute = 1;
    QN8027_SendData (0x00, QN8027_System);
  }
}

//==============================================================================
//============================== QN8027_MuteOff ================================
//==============================================================================
void QN8027_MuteOff (void)
{ //  
  if (Mute)
  { QN8027_System &= ~(1<<3);           //   Mute
    Mute = 0;
    QN8027_SendData (0x00, QN8027_System);
  }
}
//==============================================================================
//=============================== QN8027_SetCH =================================
//==============================================================================
void QN8027_SetFreqCh (void)
{ //   QN8027    QN8027_WorkCH
  QN8027_System &= 0xFC;                //   2    
  QN8027_System |= (FreqCh >> 8) & 0x03;
  QN8027_SendData (0x00, QN8027_System);
  QN8027_SendData (0x01, FreqCh);
}

/*
//==============================================================================
//=========================== QN8027_SetFrequency ==============================
//==============================================================================
void QN8027_SetFrequency (unsigned long int freq)
{ //   freq  
  FreqCh = (long)((freq - 76000)/50);     
  QN8027_SetWorkCH ();
}*/


//==============================================================================
//============================ QN8027_SetMicGain ===============================
//==============================================================================
void QN8027_SetMicGain (unsigned char micgain)
{ //   QN8027    MicGain
  static unsigned char reg_vga;
  switch (micgain)
  {// .  QN8027
    case 0:   reg_vga = 0x03;  break; // -15
    case 1:   reg_vga = 0x13;  break; // -12
    case 2:   reg_vga = 0x23;  break; // -9
    case 3:   reg_vga = 0x33;  break; // -6
    case 4:   reg_vga = 0x43;  break; // -3
    case 5:   reg_vga = 0x53;  break; // +0
    case 6:   reg_vga = 0x00;  break; // +3
    case 7:   reg_vga = 0x10;  break; // +6
    case 8:   reg_vga = 0x20;  break; // +9
    case 9:   reg_vga = 0x30;  break; // +12
    case 10:  reg_vga = 0x40;  break; // +15
    case 11:  reg_vga = 0x50;  break; // +18
    case 12:  reg_vga = 0x54;  break; // +19
    case 13:  reg_vga = 0x58;  break; // +20    
  }
  QN8027_SendData (0x04, (XSEL<<7) | reg_vga);
}

//==============================================================================
//=========================== QN8027_SetDeviation ==============================
//==============================================================================
void QN8027_SetDeviation ()
{ //    QN8027   QN8027_FDEV
  QN8027_SendData (0x11, ((FDev<<3)+FDev+FDev));  //FDev*10
}

//==============================================================================
//=========================== QN_8027_ON_OFF_RDS ===============================
//==============================================================================
void QN_8027_ON_OFF_RDS(void)
{ // / RDS (   RDS_Enable)
  QN8027_SendData (0x12, (RDS_Enable <<7)|(RDS_DEV));
}


//==============================================================================
//=========================== QN8027_SendRDSBlock ==============================
//============================================================================== 
void QN8027_SendRDSBlock (void)
{ //  1  4  RDS
  static unsigned char rdsupd1, rdsupd2;
//------------------------------------------------------------------------------
  //    RDS  QN8027
    QN8027_SendData (0x08, RDS_PI_H);
    QN8027_SendData (0x09, RDS_PI_L);
    QN8027_SendData (0x0A, RDS_GROUP | (RDS_PTY>>3)); 

    
    switch (RDS_Counter)
    { case 0: 
        QN8027_SendData (0x0B, (RDS_PTY<<5) | 0x00); 
        QN8027_SendData (0x0C, 0x00); 
        QN8027_SendData (0x0D, 0x00); 
        QN8027_SendData (0x0E, EEPROM_Read(0x00)); 
        QN8027_SendData (0x0F, EEPROM_Read(0x01)); 
        break;
//------------------------------------------------------------------------------
      case 1: 
        QN8027_SendData (0x0B, (RDS_PTY<<5) | 0x01); 
        QN8027_SendData (0x0C, 0x00); 
        QN8027_SendData (0x0D, 0x00); 
        QN8027_SendData (0x0E, EEPROM_Read(0x02)); 
        QN8027_SendData (0x0F, EEPROM_Read(0x03));  
        break;
//------------------------------------------------------------------------------
      case 2: 
        QN8027_SendData (0x0B, (RDS_PTY<<5) | 0x02); 
        QN8027_SendData (0x0C, 0x00); 
        QN8027_SendData (0x0D, 0x00); 
        QN8027_SendData (0x0E, EEPROM_Read(0x04)); 
        QN8027_SendData (0x0F, EEPROM_Read(0x05));    
        break;
//------------------------------------------------------------------------------
      case 3: 
        QN8027_SendData (0x0B, (RDS_PTY<<5) | 0x03); 
        QN8027_SendData (0x0C, 0x00); 
        QN8027_SendData (0x0D, 0x00); 
        QN8027_SendData (0x0E, EEPROM_Read(0x06)); 
        QN8027_SendData (0x0F, EEPROM_Read(0x07));  
        break;
    }
//------------------------------------------------------------------------------
    RDS_Counter++;
    if (RDS_Counter >=4)  RDS_Counter=0;
//------------------------------------------------------------------------------
    //   RDS_UPD 
    rdsupd1 = rdsupd2 = (QN8027_ReadData (0x07) & (1<<3));
//------------------------------------------------------------------------------
    //   RDSRDY.          
    if (QN8027_System & (1<<2)) QN8027_System &= ~(1<<2); // RDSRDY=0
    else QN8027_System |= (1<<2);                         // RDSRDY=1
    QN8027_SendData (0x00,  QN8027_System);    
//------------------------------------------------------------------------------
    //      RDS_UPD
    while (rdsupd1 ==  (QN8027_ReadData (0x07) & (1<<3)) )
    {}
}


//==============================================================================
//=========================== QN8027_SendRDSBlock ==============================
//============================================================================== 
void QN8027_AutoGainControl (void)
{ //   10 .
  unsigned char AudioPeak;      //    
  if (AGCEnable)
  { //            QN8027
    AudioPeak  = (QN8027_ReadData (0x07) >> 4);
     //   TXPD_CLR
    if (TxPdClr == 0)   TxPdClr = 1<<7;
    else  TxPdClr = 0;
    QN8027_SendData (0x10, (POWER | TxPdClr)); //        QN8027
    //       12*45     1 , 
    //           1      
    if (AudioPeak >AGC_MAX_VALUE)
    {// 
      AGCReturnTimeCounter = 0;
      if (AGCMicGain > 0) AGCMicGain--;
      { //     
        QN8027_SetMicGain (AGCMicGain);
      }
    }
    else if ( (AudioPeak < AGC_MAX_VALUE) && (AGCMicGain < MicGain) )
    { //    .      .    1 
      if (AGCReturnTimeCounter < 255) AGCReturnTimeCounter ++; 
      if (AGCReturnTimeCounter >= 100)
      { //  1 .    1     MicGain
          AGCMicGain++;
          QN8027_SetMicGain (AGCMicGain);
          AGCReturnTimeCounter = 0; //       
      }
    }
  }
}