/*==============================================================================
  ().........:   igor727@mail.ru
  ..: ATTINY 85
  .......: IAR Assembler for AVR 5.30.0 (5.30.0.50166)
  .........:  c    RDS ( QN8027)
  .............: 26.07.2017 .
//============================================================================*/
#include <ioavr.h>
#include <intrinsics.h>
#include "Sys.h"
#include "Settings.h"
#include "QN8027.h"
//==============================================================================
/*   EEPROM:
0x00...0x07 - RDS  -   (8 )
0x08        - RDS /:   1- , 0 - 
0x09        -  (MicGain)
0x0A        -  
0x0B        - / 
0x0C        - 
0x0D        - 
0x0E        - 
0x0F        -   
0x10...0x11 - 1 
0x12...0x13 - 2 
...
0x4A...0x4B - 30 
ADRn = 0x10 + (n-1) * 2;
//==============================================================================
//  : 
// PB0 -  
// PB1 - SDA I2C
// PB2 - SCL I2C
// PB3 -  .
// PB4 -  .
// PB5 -   */
//==============================================================================
unsigned char  CurrentState, Event;   //    .
//==============================================================================
//===============================   ================================
//==============================================================================
void main( void )
{ PORTB |= (1<<PB5);                  //  
  DDRB  = (1<<PB5)|(1<<PB0);          //  PB0  PB5  
  PRR   = 0x0B;                       //    Timer1, USI, ADC
  MCUCR = (1<<PUD)|(1<<SM1)|(0<<SM0); // Pull-Up , Sleep Mode =  PowerDown
  PCMSK = (1<<PCINT3)|(1<<PCINT4);    //     +/-
  //  Watchdog  4 .     
  MCUSR = 0;
  WDTCR = (1 << WDCE) | (1 << WDE);
  WDTCR = (1 << WDE) | (1 << WDP3); 
  //
  CurrentState = STATE_POWER_OFF;     //   
  Event = EVENT_NULL; 
//------------------------------------------------------------------------------  
  while (1)
  { 
    _delay_ms (10);   //   10 
    __watchdog_reset ();
    CountTime ();     //    (,  RDS,   ) (  Event)
    CheckButtons ();  //   (  Event)
    
//------------------------------------------------------------------------------
//----------   Event    CurrentState ---------
//------------------------------------------------------------------------------
    switch (CurrentState)
    { case STATE_POWER_OFF:           //  STATE_POWER_OFF
        switch (Event)
        {  
          case EVENT_TWO_KEY_LONG:    PowerOn ();   break;        //  
        }
        SleepIfYouCan (); //    
        break; //STATE_POWER_OFF  
//------------------------------------------------------------------------------
      case STATE_POWER_ON:            //  STATE_POWER_ON
        QN8027_AutoGainControl ();    //  
        switch (Event)
        { 
          case EVENT_KEY_UP_SHORT:    AddCode(KEY_UP);        break;  //     ( +)
          case EVENT_KEY_DOWN_SHORT:  AddCode(KEY_DOWN);      break;  //     ( -)
          case EVENT_KEY_UP_LONG:     MicSoundOff ();         break;  //   
          case EVENT_KEY_DOWN_LONG:
          case EVENT_AUTO_OFF:        PowerOff ();            break;  //  
          case EVENT_TWO_KEY_LONG:    Lock_UnLock_AddCode();  break;  // /   
        }   
        break; //STATE_POWER_ON  
//------------------------------------------------------------------------------
      case STATE_MUTE:                //   STATE_MUTE 
        switch (Event)
        { case EVENT_KEY_UP_LONG:     MicSoundOn ();      break;  //   
          case EVENT_KEY_DOWN_LONG:
          case EVENT_AUTO_OFF:        PowerOff ();        break;  //  
        }
        break; //STATE_MUTE 
//------------------------------------------------------------------------------
      case  STATE_FREQ_SET:           //   STATE_FREQ_SET
        PORTB |= 1<<PB0;              //  
        QN8027_AutoGainControl ();    //  
        switch (Event)
        { 
          case EVENT_KEY_UP_SHORT:    StepUpFreq();       break;  //    1 
          case EVENT_KEY_DOWN_SHORT:  StepDownFreq();     break;  //     1   
          case EVENT_KEY_UP_LONG:     SaveFreqToMem();    break;  //            
          case EVENT_KEY_DOWN_LONG:   NoChangeFreqOrCh(); break;  //        
          case EVENT_AUTO_OFF:        PowerOff ();        break;  //  
        } 
        break; //STATE_FREQ_SET
//------------------------------------------------------------------------------     
      case  STATE_CHANNEL_SET:        //   STATE_CHANNEL_SET
        PORTB |= 1<<PB0;              //  
        QN8027_AutoGainControl ();    // 
        switch (Event)
        { 
          case EVENT_KEY_UP_SHORT:    StepUpChannel();    break;  //         
          case EVENT_KEY_DOWN_SHORT:  StepDownChannel();  break;  //    
          case EVENT_KEY_UP_LONG:     SelectChannel();    break;  //           
          case EVENT_KEY_DOWN_LONG:   NoChangeFreqOrCh(); break;  //             
          case EVENT_AUTO_OFF:        PowerOff ();        break;  //  
        } 
        break; //STATE_CHANNEL_SET
//------------------------------------------------------------------------------    
      case  STATE_MIC_GAIN_SET:       //   STATE_MIC_GAIN_SET
        PORTB |= 1<<PB0;              //  
        switch (Event)
        { 
          case EVENT_KEY_UP_SHORT:    StepUpMicGain();    break;  //    ()
          case EVENT_KEY_DOWN_SHORT:  StepDownMicGain();  break;  //    ()
          case EVENT_KEY_UP_LONG:     SaveMicGain();      break;  //    
          case EVENT_KEY_DOWN_LONG:   NoSaveMicGain();    break;  //     
          case EVENT_AUTO_OFF:        PowerOff ();        break;  //  
        }
        break; //STATE_MIC_GAIN_SET
//------------------------------------------------------------------------------ 
      case  STATE_TURN_DEVIATION:     //   STATE_TURN_DEVIATION
        PORTB |= 1<<PB0;              //  
        switch (Event)
        { 
          case EVENT_KEY_UP_SHORT:    StepUpFDev();       break;  //    
          case EVENT_KEY_DOWN_SHORT:  StepDownFDev();     break;  //    
          case EVENT_KEY_UP_LONG:     SaveFDev();         break;  //   
          case EVENT_KEY_DOWN_LONG:   NoSaveFDev();       break;  //    
          case EVENT_AUTO_OFF:        PowerOff ();        break;  //  
        }     
        break; // STATE_TURN_DEVIATION
//------------------------------------------------------------------------------
    } // switch  
    Event = EVENT_NULL;           //  
  }// while (1)
}







